﻿using Foundation;
using MobileCoreServices;
using Plugin.FilePicker.Abstractions;
using System;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using UIKit;
using System.Diagnostics;

namespace Plugin.FilePicker
{
    /// <summary>
    /// Implementation for FilePicker
    /// </summary>
    public class FilePickerImplementation : NSObject, IUIDocumentMenuDelegate, IFilePicker
    {
        private int _requestId;
        private TaskCompletionSource<FileData> _completionSource;

        /// <summary>
        /// Event which is invoked when a file was picked
        /// </summary>
        public EventHandler<FilePickerEventArgs> Handler
        {
            get;
            set;
        }

        private void OnFilePicked(FilePickerEventArgs e)
        {
            Handler?.Invoke(null, e);
        }

        public void DidPickDocumentPicker(UIDocumentMenuViewController documentMenu, UIDocumentPickerViewController documentPicker)
        {
            documentPicker.DidPickDocument += DocumentPicker_DidPickDocument;
            documentPicker.WasCancelled += DocumentPicker_WasCancelled;
            documentPicker.DidPickDocumentAtUrls += DocumentPicker_DidPickDocumentAtUrls;
            UIApplication.SharedApplication.KeyWindow.RootViewController.PresentViewController(documentPicker, true, null);
        }

        private void DocumentPicker_DidPickDocumentAtUrls(object sender, UIDocumentPickedAtUrlsEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("URLs : " + e.Urls[0]);
            System.Diagnostics.Debug.WriteLine("URLs : " + e.Urls.Length);
            var securityEnabled = e.Urls.FirstOrDefault().StartAccessingSecurityScopedResource();

            var doc = new UIDocument(e.Urls[0]);
            System.Diagnostics.Debug.WriteLine("Documents : " + doc);

            string filename = doc.LocalizedName;

            string pathname = doc.FileUrl?.ToString();
            System.Diagnostics.Debug.WriteLine("pathnames : " + pathname);
            // iCloud drive can return null for LocalizedName.
            if (filename == null)
            {
                // Retrieve actual filename by taking the last entry after / in FileURL.
                // e.g. /path/to/file.ext -> file.ext

                // filesplit is either:
                // 0 (pathname is null, or last / is at position 0)
                // -1 (no / in pathname)
                // positive int (last occurence of / in string)
                var filesplit = pathname?.LastIndexOf('/') ?? 0;

                filename = pathname?.Substring(filesplit + 1);
            }
            System.Diagnostics.Debug.WriteLine("filenames : " + filename);

            System.Diagnostics.Debug.WriteLine("standardURL : " + e.Urls[0].StandardizedUrl);
            System.Diagnostics.Debug.WriteLine("Scheme : " + e.Urls[0].Scheme);
            System.Diagnostics.Debug.WriteLine("ResourceSpecifier : " + e.Urls[0].ResourceSpecifier);

            System.Diagnostics.Debug.WriteLine("RelativeString : " + e.Urls[0].RelativeString);
            System.Diagnostics.Debug.WriteLine("RelativePath : " + e.Urls[0].RelativePath);
            System.Diagnostics.Debug.WriteLine("Query : " + e.Urls[0].Query);
            System.Diagnostics.Debug.WriteLine("PathExtension : " + e.Urls[0].PathExtension);
            System.Diagnostics.Debug.WriteLine("Path : " + e.Urls[0].Path);


            System.Diagnostics.Debug.WriteLine("ParameterString : " + e.Urls[0].ParameterString);
            System.Diagnostics.Debug.WriteLine("LastPathComponent : " + e.Urls[0].LastPathComponent);
            System.Diagnostics.Debug.WriteLine("IsFileUrl : " + e.Urls[0].IsFileUrl);
            System.Diagnostics.Debug.WriteLine("HasDirectoryPath : " + e.Urls[0].HasDirectoryPath);
            System.Diagnostics.Debug.WriteLine("FileReferenceUrl : " + e.Urls[0].FileReferenceUrl);
            System.Diagnostics.Debug.WriteLine("FilePathUrl : " + e.Urls[0].FilePathUrl);

            System.Diagnostics.Debug.WriteLine("FileReferenceUrl : " + e.Urls[0].FileReferenceUrl);
            System.Diagnostics.Debug.WriteLine("FilePathUrl : " + e.Urls[0].FilePathUrl);

            var refurl = NSData.FromUrl(e.Urls[0].FileReferenceUrl);
            var refurldataBytes = new byte[refurl.Length];
            System.Runtime.InteropServices.Marshal.Copy(refurl.Bytes, refurldataBytes, 0, Convert.ToInt32(refurl.Length));
            System.Diagnostics.Debug.WriteLine("ref_datas : " + refurl);
            System.Diagnostics.Debug.WriteLine("ref_databytesLengths : " + refurl.Length);
            System.Diagnostics.Debug.WriteLine("ref_databytes_s : " + refurldataBytes);


            var Pathurl = NSData.FromUrl(e.Urls[0].FilePathUrl);
            var PathurldataBytes = new byte[Pathurl.Length];
            System.Runtime.InteropServices.Marshal.Copy(Pathurl.Bytes, PathurldataBytes, 0, Convert.ToInt32(Pathurl.Length));
            System.Diagnostics.Debug.WriteLine("Path_datas : " + Pathurl);
            System.Diagnostics.Debug.WriteLine("Path_databytesLengths : " + Pathurl.Length);
            System.Diagnostics.Debug.WriteLine("Path_databytes_s : " + PathurldataBytes);

            System.Diagnostics.Debug.WriteLine("BaseUrl : " + e.Urls[0].BaseUrl);
            System.Diagnostics.Debug.WriteLine("AbsoluteUrl : " + e.Urls[0].AbsoluteUrl);
            System.Diagnostics.Debug.WriteLine("AbsoluteString : " + e.Urls[0].AbsoluteString);


            var data = NSData.FromUrl(e.Urls[0]);// NSUrl.FromString(pathname));
            System.Diagnostics.Debug.WriteLine("datas : " + data);
            System.Diagnostics.Debug.WriteLine("databytesLengths : " + data.Length);
            var dataBytes = new byte[data.Length];
            System.Runtime.InteropServices.Marshal.Copy(data.Bytes, dataBytes, 0, Convert.ToInt32(data.Length));
            System.Diagnostics.Debug.WriteLine("databytes_s : " + dataBytes);

            System.Diagnostics.Debug.WriteLine("RemovePathExtension : " + e.Urls[0].RemovePathExtension());

            OnFilePicked(new FilePickerEventArgs(dataBytes, filename, pathname));
        }

        private void DocumentPicker_DidPickDocument(object sender, UIDocumentPickedEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("URL : " + e.Url);
            var securityEnabled = e.Url.StartAccessingSecurityScopedResource();
            var doc = new UIDocument(e.Url);
            System.Diagnostics.Debug.WriteLine("Document : " + doc);
            var data = NSData.FromUrl(e.Url);
            System.Diagnostics.Debug.WriteLine("data : " + data);
            System.Diagnostics.Debug.WriteLine("databytesLength : " + data.Length);
            var dataBytes = new byte[data.Length];
            System.Diagnostics.Debug.WriteLine("databytes : " + dataBytes);

            System.Runtime.InteropServices.Marshal.Copy(data.Bytes, dataBytes, 0, Convert.ToInt32(data.Length));

            string filename = doc.LocalizedName;
            System.Diagnostics.Debug.WriteLine("filename : " + filename);
            string pathname = doc.FileUrl?.ToString();
            System.Diagnostics.Debug.WriteLine("pathname : " + pathname);
            // iCloud drive can return null for LocalizedName.
            if (filename == null)
            {
                // Retrieve actual filename by taking the last entry after / in FileURL.
                // e.g. /path/to/file.ext -> file.ext

                // filesplit is either:
                // 0 (pathname is null, or last / is at position 0)
                // -1 (no / in pathname)
                // positive int (last occurence of / in string)
                var filesplit = pathname?.LastIndexOf('/') ?? 0;

                filename = pathname?.Substring(filesplit + 1);
            }

            OnFilePicked(new FilePickerEventArgs(dataBytes, filename, pathname));
        }

        /// <summary>
        /// Handles when the file picker was cancelled. Either in the
        /// popup menu or later on.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void DocumentPicker_WasCancelled(object sender, EventArgs e)
        {
            {
                var tcs = Interlocked.Exchange(ref _completionSource, null);
                tcs.SetResult(null);
            }
        }

        /// <summary>
        /// Lets the user pick a file with the systems default file picker
        /// For iOS iCloud drive needs to be configured
        /// </summary>
        /// <returns></returns>
        public async Task<FileData> PickFile()
        {
            var media = await TakeMediaAsync();

            return media;
        }

        private Task<FileData> TakeMediaAsync()
        {
            var id = GetRequestId();

            var ntcs = new TaskCompletionSource<FileData>(id);

            if (Interlocked.CompareExchange(ref _completionSource, ntcs, null) != null)
            {
                //throw new InvalidOperationException("Only one operation can be active at a time");
                System.Diagnostics.Debug.WriteLine("Only one operation can be active at a time");
                return null;
            }

            var allowedUtis = new string[] {
                //UTType.UTF8PlainText,
                //UTType.PlainText,
                //UTType.RTF,
                UTType.JPEG,
                UTType.JPEG2000,
                UTType.BMP,
                UTType.PNG,
                UTType.MPEG4,
                UTType.AppleProtectedMPEG4Video,
                UTType.AVIMovie,
                UTType.Movie,
                UTType.QuickTimeMovie,
                //UTType.Text,
                UTType.PDF,
                UTType.Image,
                //UTType.UTF16PlainText,
                //UTType.FileURL,
                UTType.Presentation,
                UTType.Spreadsheet,
                UTType.Video,
                 "com.microsoft.word.doc",
            "org.openxmlformats.wordprocessingml.document",
            "com.microsoft.powerpoint.​ppt",
            "org.openxmlformats.spreadsheetml.sheet",
            "org.openxmlformats.presentationml.presentation",
            "com.microsoft.excel.xls"
            };

            var importMenu =
                new UIDocumentMenuViewController(allowedUtis, UIDocumentPickerMode.Import)
                {
                    Delegate = this,
                    ModalPresentationStyle = UIModalPresentationStyle.Popover
                };

            UIApplication.SharedApplication.KeyWindow.RootViewController.PresentViewController(importMenu, true, null);

            var presPopover = importMenu.PopoverPresentationController;

            if (presPopover != null)
            {
                presPopover.SourceView = UIApplication.SharedApplication.KeyWindow.RootViewController.View;
                presPopover.PermittedArrowDirections = UIPopoverArrowDirection.Down;
            }

            Handler = null;

            Handler = (s, e) =>
            {
                var tcs = Interlocked.Exchange(ref _completionSource, null);

                //tcs?.SetResult(new FileData(e.FilePath, e.FileName, () => File.OpenRead(e.FilePath)));

                tcs?.SetResult(new FileData(e.FilePath, e.FileName, () =>
                {
                    var url = new Foundation.NSUrl(e.FilePath);
                    return new FileStream(url.Path, FileMode.Open, FileAccess.Read);
                }));
            };

            return _completionSource.Task;
        }

        public void WasCancelled(UIDocumentMenuViewController documentMenu)
        {
            var tcs = Interlocked.Exchange(ref _completionSource, null);

            tcs?.SetResult(null);
        }

        private int GetRequestId()
        {
            var id = _requestId;

            if (_requestId == int.MaxValue)
                _requestId = 0;
            else
                _requestId++;

            return id;
        }

        public async Task<bool> SaveFile(FileData fileToSave)
        {
            try
            {
                var documents = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                var fileName = Path.Combine(documents, fileToSave.FileName);

                File.WriteAllBytes(fileName, fileToSave.DataArray);

                return true;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                return false;
            }
        }

        public void OpenFile(NSUrl fileUrl)
        {
            var docControl = UIDocumentInteractionController.FromUrl(fileUrl);

            var window = UIApplication.SharedApplication.KeyWindow;
            var subViews = window.Subviews;
            var lastView = subViews.Last();
            var frame = lastView.Frame;

            docControl.PresentOpenInMenu(frame, lastView, true);
        }

        public void OpenFile(string fileToOpen)
        {
            var documents = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);

            var fileName = Path.Combine(documents, fileToOpen);

            if (NSFileManager.DefaultManager.FileExists(fileName))
            {
                var url = new NSUrl(fileName, true);
                OpenFile(url);
            }
        }

        public async void OpenFile(FileData fileToOpen)
        {
            var documents = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);

            var fileName = Path.Combine(documents, fileToOpen.FileName);

            if (NSFileManager.DefaultManager.FileExists(fileName))
            {
                var url = new NSUrl(fileName, true);

                OpenFile(url);
            }
            else
            {
                await SaveFile(fileToOpen);
                OpenFile(fileToOpen);
            }
        }
    }
}